% meta_ANALYSE.m This script will analyse the results of your metacognitive 
% sensitivity experiment. 

% Fn used to fit psychometric curve
Psychometric_Fit =@(g,l,u,v,x) g+(1-g-l)*0.5*(1+erf((x-u)/sqrt(2*v^2)));


%% Procedure to work out the average confidence setting, so we can do an 
% average split of confidence judgments, into low (0) and high (1). First,
% we import all the confidence ratings into an array
ALL_Confidence_Responses = Exp_Results(:,4);

% Now we calculate what the average confidence rating was for this
% participant
AVG_Confidence = mean(ALL_Confidence_Responses);

% Now we set all confidence ratings less than average to 0
ALL_Confidence_Responses(ALL_Confidence_Responses<AVG_Confidence)=0;

% Now we set all confidence setting more than 0 to 1
ALL_Confidence_Responses(ALL_Confidence_Responses>=AVG_Confidence)=1;

% Now we re-insert the adjusted confidence ratings into the results array
Exp_Results(:,4)=ALL_Confidence_Responses;

%% Procedure to fit psychometric functions to data

% Open figure window #1, and clear it, so we can plot our functions into it
figure(1)
clf

% Loop that will fit a set of functions for each experimental condition
for cond = 1 : N_Conditions
    % Find out which trial numbers in results array coinicde with this
    % experimental condition
    [idxs_Condition vals] = find(Exp_Results(:,1)==cond);

    % Identify the trial data for this condition, and store it in an array
    Cond_Results = Exp_Results(idxs_Condition,:);

    % Identify the trials that resulted in low-confidence
    [idxs_Low_Perceptual_Cat vals] = find(Cond_Results(:,3) == 0);
    % Identify the trials that resulted in high-confidence
    [idxs_High_Perceptual_Cat vals] = find(Cond_Results(:,3) == 1);
    
    % Sets confidence judgments to 0 or 1 - think this is unnecesary
    % (already done)
    [idxs_Low_Conf vals] = find(Cond_Results(:,4) == 0);   
    [idxs_High_Conf vals] = find(Cond_Results(:,4) == 1);  

    % Identify the test values that were used for this condition
    Tested_Vals = unique(Cond_Results(:,2));

    % Create an array that we will store binary perceptual categorical judgment data into 
    Results_Array_Perception = [];
    % Create an array to store data that categorise the transition from high to low confidence judgments for the lower perceptual category
    Results_Array_Confidence_Low_Category = [];
    % Create an array to store data that categorise the transition from low to high confidence judgments for the upper perceptual category
    Results_Array_Confidence_High_Category = [];

    % Loop to create our 3 datasets
    for row = 1 : size(Tested_Vals,1)

        % Identify all trials for this experimental condition that used a
        % specific test value
        [idxs vals] = find(Cond_Results(:,2) == Tested_Vals(row));

        % Store data for these trials in our Perception results array
        Results_Array_Perception(row,1)=Tested_Vals(row); % Store the test value
        Results_Array_Perception(row,2)=sum(Cond_Results(idxs,3)); % Store the sum of trials resulting in upper perceptual category responses
        Results_Array_Perception(row,3)=size(idxs,1); % Store the total number of trials for this test value

        [hmmm1 bop1] = intersect(idxs,idxs_Low_Perceptual_Cat); % Identify tests categorised in the LOW Perceptual category
        [hmmm2 bop2] = intersect(idxs,idxs_High_Conf);          % Identify tests resulting in High Confidence judgments
        [hmmm3 bop3] = intersect(hmmm1,hmmm2);                  % Identify tests categorised in the LOW Perceptual category with High Confidence

        % Store the test value
        Results_Array_Confidence_Low_Category(row,1)=Tested_Vals(row);
        if size(hmmm1,1) == 0
            % If there are no LOW categorisations - set to full number of
            % trials for this test value
            Results_Array_Confidence_Low_Category(row,2)=size(idxs,1);
        elseif size(hmmm3,1) == 0
            % No LOW categorisations w/ High Confidence - so set to full number of tests
            Results_Array_Confidence_Low_Category(row,2)=size(idxs,1);
        else
            Results_Array_Confidence_Low_Category(row,2)=size(idxs,1) - sum(Cond_Results(hmmm3,4));
        end
        Results_Array_Confidence_Low_Category(row,3)=size(idxs,1);

        [hmmm1 bop1] = intersect(idxs,idxs_High_Perceptual_Cat);
        [hmmm2 bop2] = intersect(idxs,idxs_High_Conf);
        [hmmm3 bop3] = intersect(hmmm1,hmmm2);
        Results_Array_Confidence_High_Category(row,1)=Tested_Vals(row);
        if size(hmmm3,1) == 0
            Results_Array_Confidence_High_Category(row,2)=0;
        else
            Results_Array_Confidence_High_Category(row,2)=sum(Cond_Results(hmmm3,4));
        end
        Results_Array_Confidence_High_Category(row,3)=size(idxs,1);
    end


    The_X = Results_Array_Perception(:,1);
    The_Y = Results_Array_Perception(:,2)./Results_Array_Perception(:,3);
    weights = Results_Array_Perception(:,3)./sum(Results_Array_Perception(:,3));
    % Fit a psychometric curve to perceptual categorisation data
    SPs = [0.1, 0.1, max(The_X), 100*std(The_X); % Upper limits for g, l, u ,v
    0.01, 0.01, mean(The_X), 5*std(The_X); % Start points for g, l, u ,v
    0, 0, min(The_X), std(The_X)]; % Lower limits for  g, l, u ,v

    [Psychometric, gof] = fit(The_X, The_Y, Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X) std(The_X)]);
    if abs(Psychometric.g) > 0.1 | abs(Psychometric.l) > 0.1 | Psychometric.v < 0
        accept = 0;
        try_count=0;
        while accept == 0
            try_count=try_count+1;
            [Psychometric, gof] = fit(The_X, The_Y, Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X) (5*try_count)*std(The_X)]);
            if abs(Psychometric.g) < 0.1 && abs(Psychometric.l) < 0.1 && Psychometric.v > 0 
                accept=1;
            elseif try_count == 20
                accept=1;
            end
        end
    end

    subplot(N_Conditions,1,cond);
    legend('off');
    box off;

    % Plot the results
    p = plot(Psychometric,The_X,The_Y);
    p(2).Color = [0 0 0];
    p(2).LineWidth = 3;
    legend('off');
    box off;
    hold on
    for m = 1 : numel(weights)
        if weights(m)>0
            s = scatter(The_X(m),The_Y(m),10000*weights(m),'.k');
        end
    end
    PSE_Perception = Psychometric.u(1);
    Perception_SD = abs(Psychometric.v(1));
    % Record the results
   Fit_SDs(cond,1) = Perception_SD;

    % Fit residuals analysis (is the fit much better than chance?)
    for r = 1 : size(The_X,1)
        [val idx] = find(p(2).XData>=The_X(r),1);
        FitSumResiduals(r) = abs(The_Y(r) - p(2).YData(idx));
        ChanceSumResiduals(r) = abs(mean(The_Y) - p(2).YData(idx));
    end
    FitSumResiduals = sum(FitSumResiduals);
    ChanceSumResiduals = sum(ChanceSumResiduals);
    FitRatio = ChanceSumResiduals/FitSumResiduals;
    if FitRatio <= 1.5
        % Fit to data is equal to or worse than chance
        set(gca,'Color',[1 0 0 0.1]);
        p(2).Color = [1 0 0];
    end

    The_X = Results_Array_Confidence_Low_Category(:,1);
    The_Y = Results_Array_Confidence_Low_Category(:,2)./Results_Array_Confidence_Low_Category(:,3); 
    weights = Results_Array_Confidence_Low_Category(:,3)./sum(Results_Array_Confidence_Low_Category(:,3));
    [Psychometric2,gof] = fit(The_X,The_Y,Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X)-std(The_X) std(The_X)]);
    if abs(Psychometric2.g) > 0.1 | abs(Psychometric2.l) > 0.1 | Psychometric2.v < 0
        accept = 0;
        try_count=0;
        while accept == 0
            try_count=try_count+1;
            [Psychometric2, gof] = fit(The_X, The_Y, Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X)-std(The_X) (5*try_count)*std(The_X)]);
            if abs(Psychometric2.g) < 0.1 && abs(Psychometric2.l) < 0.1 && Psychometric2.v > 0 
                accept=1;
            elseif try_count == 20
                accept=1;
            end
        end
    end

    h2 = plot(Psychometric2,The_X,The_Y);
    legend('off');
    box off;
    h2(2).Color = [0 0 1];
    h2(2).LineWidth = 3;
    for m = 1 : numel(weights)
        if weights(m)>0
            s = scatter(The_X(m),The_Y(m),10000*weights(m),'.b');
        end
    end
    Lower_Conf_SD = abs(Psychometric2.v(1));

    % Fit residuals analysis (is the fit much better than chance?)
    TheXX = find(The_X <= PSE_Perception);
    TheYY = The_Y(TheXX);
    TheXX = The_X(TheXX);
    for r = 1 : size(TheXX,1)
        [val idx] = find(h2(2).XData>=TheXX(r),1);
        FitSumResiduals(r) = abs(TheYY(r) - h2(2).YData(idx));
        ChanceSumResiduals(r) = abs(mean(The_Y) - h2(2).YData(idx));
    end
    FitSumResiduals = sum(FitSumResiduals);
    ChanceSumResiduals = sum(ChanceSumResiduals);
    FitRatio = ChanceSumResiduals/FitSumResiduals;
    if FitRatio <= 1.5
        % Fit to data is equal to or worse than chance
        set(gca,'Color',[1 0 0 0.1]);
        h2(2).Color = [1 0 0];
    end
    
    The_X = Results_Array_Confidence_High_Category(:,1);
    The_Y = Results_Array_Confidence_High_Category(:,2)./Results_Array_Confidence_High_Category(:,3); 
    weights = Results_Array_Confidence_High_Category(:,3)./sum(Results_Array_Confidence_High_Category(:,3));
    [Psychometric3,gof] = fit(The_X,The_Y,Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X)+std(The_X) std(The_X)]);
    if abs(Psychometric3.g) > 0.1 | abs(Psychometric3.l) > 0.1 | Psychometric3.v < 0
        accept = 0;
        try_count=0;
        while accept == 0
            try_count=try_count+1;
            [Psychometric3, gof] = fit(The_X, The_Y, Psychometric_Fit,'Weight',weights,'StartPoint',[0 0 mean(The_X)+std(The_X) (5*try_count)*std(The_X)]);
            if abs(Psychometric3.g) < 0.1 && abs(Psychometric3.l) < 0.1 && Psychometric3.v > 0 
                accept=1;
            elseif try_count == 20
                accept=1;
            end
        end
    end
    h3 = plot(Psychometric3,The_X,The_Y);
    legend('off');
    box off;
    mColor = [0 0.5 0];
    h3(2).Color = mColor;
    h3(2).LineWidth = 3;
    for m = 1 : numel(weights)
        if weights(m)>0
            s = scatter(The_X(m),The_Y(m),10000*weights(m),'.');
            s.CData = mColor;
        end
    end
    Upper_Conf_SD = abs(Psychometric3.v(1));

    % Fit residuals analysis (is the fit much better than chance?)
    TheXX = find(The_X >= PSE_Perception);
    TheYY = The_Y(TheXX);
    TheXX = The_X(TheXX);
    for r = 1 : size(TheXX,1)
        [val idx] = find(h3(2).XData>=TheXX(r),1);
        FitSumResiduals(r) = abs(TheYY(r) - h3(2).YData(idx));
        ChanceSumResiduals(r) = abs(mean(The_Y) - h3(2).YData(idx));
    end
    FitSumResiduals = sum(FitSumResiduals);
    ChanceSumResiduals = sum(ChanceSumResiduals);
    FitRatio = ChanceSumResiduals/FitSumResiduals;
    if FitRatio <= 1.5
        % Fit to data is equal to or worse than chance
        set(gca,'Color',[1 0 0 0.1]);
        h3(2).Color = [1 0 0];
    end

    Fit_SDs(cond,2) = mean([Lower_Conf_SD Upper_Conf_SD]);

    xlabel('Test Vales');
    ylabel('Pr. Responses');
    set(gca,'FontSize',14)
    legend('off');
    box off;
end

Fit_Ratios = Fit_SDs(:,2)./Fit_SDs(:,1);
