% meta_SETUP.m This script will setup an experiment to assess metacognitive 
% sensitivity. Before you call it, you need to define at least 6 variables 
% (and you may opt to define an additional two). In combination these will 
% describe and control your experiment. Detailed descriptions of these 
% variables are below. You can cut and paste these comments into your 
% experimental code as a reminder.

% N_Conditions = number of experimental conditions (minimum 1). 

% Guess_Low_Asymptote = Guess what test value will result in 10% of tests
% being categorised as beloning to the second perceptual category (i.e. as
% right tilted in a subjective binary categorisation task), or in 55% of
% tests being correctly detected in two (or more) alternative-forced-choice
% tasks. 

% Guess_Mean = Guess what test value will correspond with the mean response
% (i.e. with the point of subjective binary categorisation tasks, or the
% detection threshold in two (or more) alternative-forced-choice tasks. 

% Guess_High_Asymptote = Guess what test value will result in 90% of tests
% being categorised as belonging to the second perceptual category (i.e. as
% right tilted in a subjective binary categorisation task), or in 95% of
% tests being correctly detected in two (or more) alternative-forced-choice
% tasks. 

% N_Trials_Per_Condition = Specify the number of trials you want to run for 
% each of your experimental conditions. We recommend a minimuim of 50. 

% Min_Step_Size = Specify the minimum step size you want to use to adjust 
% your test values. Make sure this is small enough to accurately measure 
% sensitivity to your tests, but not so small that you are not meaningfully 
% adjusting test values on successive trials. Also, make sure you can 
% physically adjust your test values by this value, and by 2x and by 4x 
% this value

% Test_Limit_Low = OPTIONAL variable. Define this variable if you want
% to constrain your test values to a minimum value. For instance, you may 
% want to constrain your test values to a minimum value that can be output 
% by your testing apparatus (e.g. a contrast value of 0)

% Test_Limit_High = OPTIONAL variable. Define this variable if you want
% to constrain your test values to a maximum value. For instance, you may 
% want to constrain your test values to a maximum value that can be output 
% by your testing apparatus (e.g. a contrast value of 100%)


chk = round((Guess_Mean-Guess_Low_Asymptote)/Min_Step_Size);
if chk < 2
    error('Insufficient Test Range Recommended: Please decrease minimum stepsize')
end
chk = round((Guess_High_Asymptote-Guess_Mean)/Min_Step_Size);
if chk < 2
    error('Insufficient Test Range Recommended: Please increase high test limit')
end

if ~exist('Test_Limit_Low')
    Test_Limit_Low = NaN;
end
if ~exist('Test_Limit_High')
    Test_Limit_High = NaN;
end

First_Guesses = [min(Guess_Mean-(chk*Min_Step_Size),Test_Limit_Low) Guess_Mean Guess_Mean+(chk*Min_Step_Size)];
Test_Range = (First_Guesses(3)-First_Guesses(1))*3;
Test_Inputs = max([-Test_Range/2 Test_Limit_Low]) :Min_Step_Size: max([Test_Range/2 Test_Limit_High]);
N_Test_Levels = size(Test_Inputs,2);

Staircase_Sample_Pts = [First_Guesses(1) First_Guesses(1)+(round(chk/2)*Min_Step_Size) First_Guesses(2) First_Guesses(2)+(round(chk/2)*Min_Step_Size) First_Guesses(3)];
if N_Conditions > 1
    for Row = 2 : N_Conditions
        Staircase_Sample_Pts(Row,:)=Staircase_Sample_Pts(Row-1,:);
    end
end

Staircase_Histories = zeros(size(Staircase_Sample_Pts,2),10,N_Conditions);
Staircase_Histories(3:5,:,:) = 1;

Trial_Counts = zeros(N_Conditions,5);

Reversals = zeros(N_Conditions,size(Staircase_Sample_Pts,2));

Random_Trial_Order = randperm(N_Conditions*N_Trials_Per_Condition);
%Tested_Level_Counts = zeros(N_Test_Levels,N_Conditions);

Test_Number=1;

Condition_N = ceil(Random_Trial_Order(Test_Number)/N_Trials_Per_Condition);

if Condition_N == 1
    Adj_Test_Number = 0;
else
    Adj_Test_Number = -(Condition_N-1)*N_Trials_Per_Condition;
end

if Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.05
    % Target lower performance asymptope - to ensure performance varies
    % from near 100% Lower categorisations...
    Staircase_N = 1;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.05 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.35
    % Target lower confidence boundary
    Staircase_N = 2;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.35 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.65
    % Target perceptual category boundary
    Staircase_N = 3;
elseif Random_Trial_Order(Test_Number)-Adj_Test_Number >= N_Trials_Per_Condition*0.65 && Random_Trial_Order(Test_Number)-Adj_Test_Number < N_Trials_Per_Condition*0.95
    % Target upper confidence boundary
    Staircase_N = 4;
else
    % Target upper performance asymptope - to ensure performance varies
    % upto near 100% Upper categorisations
    Staircase_N = 5;
end
[~,Test_Level] = find(Test_Inputs == Staircase_Sample_Pts(Condition_N,Staircase_N));

Test_Val = Staircase_Sample_Pts(Condition_N,Staircase_N);

N_Trials = N_Conditions*N_Trials_Per_Condition;

